@@ -1,6 +1,6 @@
 #![recursion_limit = "128"]
 #![cfg_attr(feature = "extension-module", crate_type = "cdylib")]
-#![allow(unused_imports, unused_unsafe, unused_variables)]
+#![allow(unused_imports, unused_variables)]
 
 extern crate fastobo;
 extern crate pyo3;
@@ -8,10 +8,6 @@ extern crate pyo3;
 extern crate pyo3_built;
 extern crate libc;
 extern crate pest;
-extern crate url;
-#[macro_use]
-extern crate opaque_typedef_macros;
-extern crate opaque_typedef;
 #[macro_use]
 extern crate fastobo_py_derive_internal;
 extern crate fastobo_graphs;
