@@ -5,13 +5,12 @@ use std::fmt::Write;
 use std::str::FromStr;
 
 use pyo3::class::basic::CompareOp;
-use pyo3::exceptions::TypeError;
-use pyo3::exceptions::ValueError;
+use pyo3::exceptions::PyTypeError;
+use pyo3::exceptions::PyValueError;
 use pyo3::prelude::*;
 use pyo3::types::PyAny;
 use pyo3::types::PyString;
 use pyo3::AsPyPointer;
-use pyo3::AsPyRef;
 use pyo3::PyNativeType;
 use pyo3::PyObjectProtocol;
 use pyo3::PyTypeInfo;
@@ -27,13 +26,12 @@ use crate::utils::AbstractClass;
 
 // --- Module export ----------------------------------------------------------
 
-#[pymodule(id)]
+#[pymodule]
+#[pyo3(name = "id")]
 pub fn init(_py: Python, m: &PyModule) -> PyResult<()> {
     m.add_class::<self::BaseIdent>()?;
     m.add_class::<self::PrefixedIdent>()?;
     m.add_class::<self::UnprefixedIdent>()?;
-    m.add_class::<self::IdentPrefix>()?;
-    m.add_class::<self::IdentLocal>()?;
     m.add_class::<self::Url>()?;
     m.add("__name__", "fastobo.id")?;
 
@@ -61,13 +59,14 @@ pub fn init(_py: Python, m: &PyModule) -> PyResult<()> {
     ///     >>> fastobo.id.parse("http://purl.obolibrary.org/obo/IAO_0000231")
     ///     Url('http://purl.obolibrary.org/obo/IAO_0000231')
     ///
-    #[pyfn(m, "parse")]
+    #[pyfn(m)]
+    #[pyo3(name = "parse")]
     fn parse(py: Python, s: &str) -> PyResult<Ident> {
         match fastobo::ast::Ident::from_str(s) {
-            Ok(id) => Ok(Ident::from_py(id, py)),
+            Ok(id) => Ok(id.into_py(py)),
             Err(e) => {
                 let err = PyErr::from(Error::from(e));
-                raise!(py, ValueError("could not parse identifier") from err)
+                raise!(py, PyValueError("could not parse identifier") from err)
             }
         }
     }
@@ -92,10 +91,11 @@ pub fn init(_py: Python, m: &PyModule) -> PyResult<()> {
     ///     True
     ///     >>> fastobo.id.is_valid("definitely not an identifier")
     ///     False
-    #[pyfn(m, "is_valid")]
+    #[pyfn(m)]
+    #[pyo3(name = "is_valid")]
     fn is_valid(_py: Python, s: &str) -> bool {
         let rule = fastobo::syntax::Rule::Id;
-        match fastobo::syntax::Lexer::parse(rule, s) {
+        match fastobo::syntax::Lexer::tokenize(rule, s) {
             Err(e) => false,
             Ok(pairs) => pairs.as_str().len() == s.len(),
         }
@@ -108,16 +108,16 @@ pub fn init(_py: Python, m: &PyModule) -> PyResult<()> {
 
 macro_rules! impl_convert {
     ($base:ident, $cls:ident) => {
-        impl FromPy<$crate::fastobo::ast::$base> for $cls {
-            fn from_py(id: $crate::fastobo::ast::$base, py: Python) -> $cls {
-                let ident: ast::Ident = id.into();
-                $cls::from_py(ident, py)
+        impl IntoPy<$cls> for $crate::fastobo::ast::$base {
+            fn into_py(self, py: Python) -> $cls {
+                let ident: $crate::fastobo::ast::Ident = self.into();
+                ident.into_py(py)
             }
         }
 
-        impl FromPy<$cls> for $crate::fastobo::ast::$base {
-            fn from_py(id: $cls, py: Python) -> $crate::fastobo::ast::$base {
-                let ident: ast::Ident = id.into_py(py);
+        impl IntoPy<$crate::fastobo::ast::$base> for $cls {
+            fn into_py(self, py: Python) -> $crate::fastobo::ast::$base {
+                let ident: $crate::fastobo::ast::Ident = self.into_py(py);
                 $crate::fastobo::ast::$base::from(ident)
             }
         }
@@ -145,46 +145,30 @@ impl Display for Ident {
     }
 }
 
-impl FromPy<fastobo::ast::Ident> for Ident {
-    fn from_py(ident: fastobo::ast::Ident, py: Python) -> Self {
-        match ident {
-            ast::Ident::Unprefixed(id) => {
-                Py::new(py, UnprefixedIdent::new(id))
-                    .map(Ident::Unprefixed)
-                    .expect("could not allocate on Python heap")
-            }
-            ast::Ident::Prefixed(id) => {
-                Py::new(py, PrefixedIdent::from_py(id, py))
-                    .map(Ident::Prefixed)
-                    .expect("could not allocate on Python heap")
-            }
-            ast::Ident::Url(id) => {
-                Py::new(py, Url::from_py(id, py))
-                    .map(Ident::Url)
-                    .expect("could not allocate on Python heap")
-            }
-        }
+impl IntoPy<Ident> for fastobo::ast::Ident {
+    fn into_py(self, py: Python) -> Ident {
+        match self {
+            ast::Ident::Unprefixed(id) => Py::new(py, id.into_py(py)).map(Ident::Unprefixed),
+            ast::Ident::Prefixed(id) => Py::new(py, id.into_py(py)).map(Ident::Prefixed),
+            ast::Ident::Url(id) => Py::new(py, id.into_py(py)).map(Ident::Url),
+        }.expect("could not allocate on Python heap")
     }
 }
 
-impl FromPy<Ident> for fastobo::ast::Ident {
-    fn from_py(ident: Ident, py: Python) -> Self {
-        match ident {
+impl IntoPy<fastobo::ast::Ident> for Ident {
+    fn into_py(self, py: Python) -> fastobo::ast::Ident {
+        match self {
             Ident::Unprefixed(id) => {
                 let i = id.as_ref(py).borrow();
-                ast::Ident::Unprefixed((*i).inner.clone())
+                ast::Ident::from((*i).inner.clone())
             }
             Ident::Prefixed(id) => {
                 let i = id.as_ref(py).borrow();
-                let p = i.prefix.as_ref(py).borrow();
-                let l = i.local.as_ref(py).borrow();
-                ast::Ident::Prefixed(
-                    ast::PrefixedIdent::new((*p).inner.clone(), (*l).inner.clone())
-                )
+                ast::Ident::from((*i).inner.clone())
             }
             Ident::Url(id) => {
                 let i = id.as_ref(py).borrow();
-                ast::Ident::Url((*i).inner.clone())
+                ast::Ident::from((*i).inner.clone())
             }
         }
     }
@@ -217,83 +201,70 @@ impl AbstractClass for BaseIdent {
 /// Example:
 ///     >>> ident = fastobo.id.PrefixedIdent('GO', '0009637')
 ///     >>> ident.prefix
-///     IdentPrefix('GO')
+///     'GO'
 ///     >>> ident.local
-///     IdentLocal('0009637')
+///     '0009637'
 ///     >>> str(ident)
 ///     'GO:0009637'
 ///
 #[pyclass(extends=BaseIdent, module="fastobo.id")]
-#[derive(Debug, FinalClass)]
+#[derive(Debug, FinalClass, Clone, PartialEq, Eq)]
+#[base(BaseIdent)]
 pub struct PrefixedIdent {
-    prefix: Py<IdentPrefix>,
-    local: Py<IdentLocal>,
+    inner: ast::PrefixedIdent,
 }
 
 impl PrefixedIdent {
-    fn new(prefix: Py<IdentPrefix>, local: Py<IdentLocal>) -> Self {
-        PrefixedIdent { prefix, local }
+    fn new(prefix: &str, local: &str) -> Self {
+        PrefixedIdent {
+            inner: ast::PrefixedIdent::new(prefix, local)
+        }
     }
 }
 
 impl ClonePy for PrefixedIdent {
-    fn clone_py(&self, py: Python) -> Self {
-        Self {
-            prefix: self.prefix.clone_ref(py),
-            local: self.local.clone_ref(py),
-        }
+    fn clone_py(&self, _py: Python) -> Self {
+        self.clone()
     }
 }
 
 impl Display for PrefixedIdent {
     fn fmt(&self, f: &mut Formatter) -> FmtResult {
-        let gil = Python::acquire_gil();
-        let py = gil.python();
-
-        let p = self.prefix.as_ref(py).borrow().inner.clone();
-        let l = self.local.as_ref(py).borrow().inner.clone();
-        fastobo::ast::PrefixedIdent::new(p, l).fmt(f)
+        self.inner.fmt(f)
     }
 }
 
-impl PartialEq for PrefixedIdent {
-    fn eq(&self, other: &Self) -> bool {
-        let gil = Python::acquire_gil();
-        let py = gil.python();
-
-        let res = *self.prefix.as_ref(py).borrow() == *other.prefix.as_ref(py).borrow()
-            && *self.local.as_ref(py).borrow() == *other.local.as_ref(py).borrow();
-
-        res
+impl From<ast::PrefixedIdent> for PrefixedIdent {
+    fn from(id: ast::PrefixedIdent) -> Self {
+        Self {
+            inner: id
+        }
     }
 }
 
-impl Eq for PrefixedIdent {}
-
-impl FromPy<PrefixedIdent> for ast::PrefixedIdent {
-    fn from_py(ident: PrefixedIdent, py: Python) -> Self {
-        ast::PrefixedIdent::new(
-            ident.prefix.as_ref(py).borrow().clone(),
-            ident.local.as_ref(py).borrow().clone(),
-        )
+impl From<PrefixedIdent> for ast::PrefixedIdent {
+    fn from(id: PrefixedIdent) -> Self {
+        id.inner
     }
 }
 
-impl FromPy<PrefixedIdent> for ast::Ident {
-    fn from_py(ident: PrefixedIdent, py: Python) -> Self {
-        Self::from(ast::PrefixedIdent::from_py(ident, py))
+impl IntoPy<ast::PrefixedIdent> for PrefixedIdent {
+    fn into_py(self, py: Python) -> ast::PrefixedIdent {
+        self.inner
     }
 }
 
-impl FromPy<ast::PrefixedIdent> for PrefixedIdent {
-    fn from_py(id: ast::PrefixedIdent, py: Python) -> Self {
-        let prefix: IdentPrefix = id.prefix().clone().into();
-        let local: IdentLocal = id.local().clone().into();
+impl IntoPy<ast::Ident> for PrefixedIdent {
+    fn into_py(self, _py: Python) -> ast::Ident {
+        ast::Ident::from(self.inner)
+    }
+}
 
-        Self::new(
-            Py::new(py, prefix).expect("could not allocate on Python heap"),
-            Py::new(py, local).expect("could not allocate on Python heap"),
-        )
+impl IntoPy<PrefixedIdent> for ast::PrefixedIdent {
+    fn into_py(self, _py: Python) -> PrefixedIdent {
+        PrefixedIdent {
+            inner: self
+        }
     }
 }
 
@@ -305,104 +276,54 @@ impl PrefixedIdent {
     /// escaping will occur when serializing this identifier.
     ///
     /// Arguments:
-    ///     prefix (str or `IdentPrefix`): the idspace of the identifier.
-    ///     local (str or `IdentLocal`): the local part of the identifier.
+    ///     prefix (str): the idspace of the identifier.
+    ///     local (str): the local part of the identifier.
     ///
     #[new]
-    fn __init__(prefix: &PyAny, local: &PyAny) -> PyResult<PyClassInitializer<Self>> {
-        let gil = Python::acquire_gil();
-        let py = gil.python();
-
-        let p = if let Ok(prefix) = prefix.extract::<Py<IdentPrefix>>() {
-            prefix.clone_ref(py)
-        } else if let Ok(ref s) = PyString::try_from(prefix) {
-            let string = s.to_string();
-            Py::new(py, IdentPrefix::new(ast::IdentPrefix::new(string)))?
-        } else {
-            let ty = prefix.get_type().name();
-            let msg = format!("expected IdentPrefix or str, found {}", ty);
-            return TypeError::into(msg);
-        };
-
-        let l = if let Ok(local) = local.extract::<Py<IdentLocal>>() {
-            local.clone_ref(py)
-        } else if let Ok(ref s) = PyString::try_from(local) {
-            let string = s.to_string();
-            Py::new(py, IdentLocal::new(ast::IdentLocal::new(string)))?
-        } else {
-            let ty = local.get_type().name();
-            let msg = format!("expected IdentLocal or str, found {}", ty);
-            return TypeError::into(msg);
-        };
-
+    fn __init__(prefix: &str, local: &str) -> PyResult<PyClassInitializer<Self>> {
         Ok(
             PyClassInitializer::from(BaseIdent {})
-                .add_subclass(Self::new(p, l))
+                .add_subclass(Self::new(prefix, local))
         )
     }
 
     /// `~fastobo.id.IdentPrefix`: the IDspace of the identifier.
     #[getter]
-    fn get_prefix<'py>(&self, py: Python<'py>) -> PyResult<Py<IdentPrefix>> {
-        Ok(self.prefix.clone_ref(py))
+    fn get_prefix<'py>(&self) -> &str {
+        self.inner.prefix()
     }
 
     #[setter]
-    fn set_prefix(&mut self, prefix: &PyAny) -> PyResult<()> {
-        let py = prefix.py();
-        self.prefix = if let Ok(prefix) = prefix.extract::<Py<IdentPrefix>>() {
-            prefix.clone_ref(py)
-        } else if let Ok(ref s) = PyString::try_from(prefix) {
-            let string = s.to_string();
-            Py::new(py, IdentPrefix::new(ast::IdentPrefix::new(string)))?
-        } else {
-            let ty = prefix.get_type().name();
-            let msg = format!("expected IdentPrefix or str, found {}", ty);
-            return TypeError::into(msg);
-        };
-        Ok(())
+    fn set_prefix(&mut self, prefix: &str) {
+        self.inner = ast::PrefixedIdent::new(
+            prefix,
+            self.inner.local()
+        );
     }
 
     /// `~fastobo.id.IdentLocal`: the local part of the identifier.
     #[getter]
-    fn get_local<'py>(&self, py: Python<'py>) -> PyResult<Py<IdentLocal>> {
-        Ok(self.local.clone_ref(py))
+    fn get_local<'py>(&self) -> &str {
+        self.inner.local()
     }
 
     #[setter]
-    fn set_local(&mut self, local: &PyAny) -> PyResult<()> {
-        let py = local.py();
-        self.local = if let Ok(local) = local.extract::<Py<IdentLocal>>() {
-            local.clone_ref(py)
-        } else if let Ok(ref s) = PyString::try_from(local) {
-            let string = s.to_string();
-            Py::new(py, IdentLocal::new(ast::IdentLocal::new(string)))?
-        } else {
-            let ty = local.get_type().name();
-            let msg = format!("expected IdentLocal or str, found {}", ty);
-            return TypeError::into(msg);
-        };
-        Ok(())
+    fn set_local(&mut self, local: &str) {
+        self.inner = ast::PrefixedIdent::new(
+            self.inner.prefix(),
+            local
+        );
     }
 }
 
 #[pyproto]
 impl PyObjectProtocol for PrefixedIdent {
+    fn __hash__(&self) -> u64 {
+        impl_hash!(self.inner.prefix(), ":", self.inner.local())
+    }
+
     fn __repr__(&self) -> PyResult<PyObject> {
-        // acquire the GIL
-        let gil = Python::acquire_gil();
-        let py = gil.python();
-        // extract inner references
-        let pref = self.prefix.as_ref(py);
-        let lref = self.local.as_ref(py);
-        // get the formatted `repr` string
-        let fmt = PyString::new(py, "PrefixedIdent({!r}, {!r})");
-        let repr = fmt.call_method1(
-            "format",
-            (pref.borrow().inner.as_str(), lref.borrow().inner.as_str())
-        );
-        // convert to an object before releasing the GIL
-        Ok(repr?.to_object(py))
+        impl_repr!(self, PrefixedIdent(self.inner.prefix(), self.inner.local()))
     }
 
     fn __str__(&self) -> PyResult<String> {
@@ -413,10 +334,10 @@ impl PyObjectProtocol for PrefixedIdent {
         let py = other.py();
         if let Ok(r) = other.extract::<Py<PrefixedIdent>>() {
             let r = r.as_ref(py).borrow();
-            let lp = &*self.prefix.as_ref(py).borrow();
-            let ll = &*self.local.as_ref(py).borrow();
-            let rp = &*r.prefix.as_ref(py).borrow();
-            let rl = &*r.local.as_ref(py).borrow();
+            let lp = self.inner.prefix();
+            let ll = self.inner.local();
+            let rp = r.inner.prefix();
+            let rl = r.inner.local();
             match op {
                 CompareOp::Eq => Ok((lp, ll) == (rp, rl)),
                 CompareOp::Ne => Ok((lp, ll) != (rp, rl)),
@@ -430,9 +351,9 @@ impl PyObjectProtocol for PrefixedIdent {
                 CompareOp::Eq => Ok(false),
                 CompareOp::Ne => Ok(true),
                 _ => {
-                    let n = other.get_type().name();
+                    let n = other.get_type().name()?;
                     let msg = format!("expected PrefixedIdent, found {}", n);
-                    TypeError::into(msg)
+                    Err(PyTypeError::new_err(msg))
                 }
             }
         }
@@ -452,7 +373,8 @@ impl PyObjectProtocol for PrefixedIdent {
 ///     hello world
 ///
 #[pyclass(extends=BaseIdent, module="fastobo.id")]
-#[derive(Clone, Debug, Eq, Hash, OpaqueTypedef, PartialEq, FinalClass)]
+#[derive(Clone, Debug, Eq, Hash, PartialEq, FinalClass)]
+#[base(BaseIdent)]
 pub struct UnprefixedIdent {
     inner: ast::UnprefixedIdent,
 }
@@ -487,21 +409,21 @@ impl From<UnprefixedIdent> for ast::UnprefixedIdent {
     }
 }
 
-impl FromPy<UnprefixedIdent> for ast::UnprefixedIdent {
-    fn from_py(id: UnprefixedIdent, _py: Python) -> Self {
-        Self::from(id)
+impl IntoPy<ast::UnprefixedIdent> for UnprefixedIdent {
+    fn into_py(self, py: Python) -> ast::UnprefixedIdent {
+        ast::UnprefixedIdent::from(self)
     }
 }
 
 impl From<UnprefixedIdent> for ast::Ident {
     fn from(id: UnprefixedIdent) -> Self {
-        ast::Ident::Unprefixed(ast::UnprefixedIdent::from(id))
+        ast::Ident::from(ast::UnprefixedIdent::from(id))
     }
 }
 
-impl FromPy<UnprefixedIdent> for ast::Ident {
-    fn from_py(id: UnprefixedIdent, _py: Python) -> Self {
-        Self::from(id)
+impl IntoPy<ast::Ident> for UnprefixedIdent {
+    fn into_py(self, py: Python) -> ast::Ident {
+        ast::Ident::from(self)
     }
 }
 
@@ -511,9 +433,9 @@ impl From<ast::UnprefixedIdent> for UnprefixedIdent {
     }
 }
 
-impl FromPy<ast::UnprefixedIdent> for UnprefixedIdent {
-    fn from_py(id: ast::UnprefixedIdent, _py: Python) -> Self {
-        Self::from(id)
+impl IntoPy<UnprefixedIdent> for ast::UnprefixedIdent {
+    fn into_py(self, py: Python) -> UnprefixedIdent {
+        UnprefixedIdent::from(self)
     }
 }
 
@@ -545,11 +467,12 @@ impl UnprefixedIdent {
 
 #[pyproto]
 impl PyObjectProtocol for UnprefixedIdent {
+    fn __hash__(&self) -> u64 {
+        impl_hash!(self.inner.as_str())
+    }
+
     fn __repr__(&self) -> PyResult<PyObject> {
-        let gil = Python::acquire_gil();
-        let py = gil.python();
-        let fmt = PyString::new(py, "UnprefixedIdent({!r})").to_object(py);
-        fmt.call_method1(py, "format", (self.inner.as_str(),))
+        impl_repr!(self, UnprefixedIdent(self.inner.as_str()))
     }
 
     fn __str__(&'p self) -> PyResult<&'p str> {
@@ -572,9 +495,9 @@ impl PyObjectProtocol for UnprefixedIdent {
                 CompareOp::Eq => Ok(false),
                 CompareOp::Ne => Ok(true),
                 _ => {
-                    let n = other.get_type().name();
+                    let n = other.get_type().name()?;
                     let msg = format!("expected UnprefixedIdent, found {}", n);
-                    TypeError::into(msg)
+                    Err(PyTypeError::new_err(msg))
                 }
             }
         }
@@ -598,14 +521,14 @@ impl PyObjectProtocol for UnprefixedIdent {
 ///     ValueError: invalid url: ...
 ///
 #[pyclass(extends=BaseIdent, module="fastobo.id")]
-#[derive(Clone, ClonePy, Debug, Eq, Hash, OpaqueTypedef, Ord, PartialEq, PartialOrd, FinalClass)]
-#[opaque_typedef(derive(FromInner, IntoInner))]
+#[derive(Clone, ClonePy, Debug, Eq, Hash, Ord, PartialEq, PartialOrd, FinalClass)]
+#[base(BaseIdent)]
 pub struct Url {
-    inner: url::Url,
+    inner: ast::Url,
 }
 
 impl Url {
-    pub fn new(url: url::Url) -> Self {
+    pub fn new(url: ast::Url) -> Self {
         Self { inner: url }
     }
 }
@@ -616,27 +539,40 @@ impl Display for Url {
     }
 }
 
-impl FromPy<url::Url> for Url {
-    fn from_py(url: url::Url, _py: Python) -> Self {
+impl From<ast::Url> for Url {
+    fn from(url: ast::Url) -> Self {
         Self::new(url)
     }
 }
 
+
+impl From<Url> for fastobo::ast::Url {
+    fn from(url: Url) -> Self {
+        url.inner
+    }
+}
+
 impl From<Url> for fastobo::ast::Ident {
     fn from(url: Url) -> Self {
-        fastobo::ast::Ident::Url(url.inner)
+        fastobo::ast::Ident::from(url.inner)
     }
 }
 
-impl FromPy<Url> for fastobo::ast::Ident {
-    fn from_py(url: Url, _py: Python) -> Self {
-        Self::from(url)
+impl IntoPy<Url> for ast::Url {
+    fn into_py(self, _py: Python) -> Url {
+        Url::new(self)
     }
 }
 
-impl FromPy<Url> for url::Url {
-    fn from_py(url: Url, _py: Python) -> Self {
-        url.inner
+impl IntoPy<fastobo::ast::Ident> for Url {
+    fn into_py(self, _py: Python) -> fastobo::ast::Ident {
+        fastobo::ast::Ident::from(self)
+    }
+}
+
+impl IntoPy<ast::Url> for Url {
+    fn into_py(self, _py: Python) -> ast::Url {
+        self.inner
     }
 }
 
@@ -653,20 +589,21 @@ impl Url {
     #[new]
     fn __new__(value: &str) -> PyResult<PyClassInitializer<Self>> {
         let init = PyClassInitializer::from(BaseIdent {});
-        match url::Url::from_str(value) {
+        match ast::Url::from_str(value) {
             Ok(url) => Ok(init.add_subclass(Url::new(url))),
-            Err(e) => ValueError::into(format!("invalid url: {}", e)),
+            Err(e) => Err(PyValueError::new_err(format!("invalid url: {}", e))),
         }
     }
 }
 
 #[pyproto]
 impl PyObjectProtocol for Url {
+    fn __hash__(&self) -> u64 {
+        impl_hash!(self.inner.as_str())
+    }
+
     fn __repr__(&self) -> PyResult<PyObject> {
-        let gil = Python::acquire_gil();
-        let py = gil.python();
-        let fmt = PyString::new(py, "Url({!r})").to_object(py);
-        fmt.call_method1(py, "format", (self.inner.as_str(),))
+        impl_repr!(self, Url(self.inner.as_str()))
     }
 
     /// Retrieve the URL in a serialized form.
@@ -691,134 +628,11 @@ impl PyObjectProtocol for Url {
                 CompareOp::Eq => Ok(false),
                 CompareOp::Ne => Ok(true),
                 _ => {
-                    let n = other.get_type().name();
+                    let n = other.get_type().name()?;
                     let msg = format!("expected str or Url, found {}", n);
-                    TypeError::into(msg)
+                    Err(PyTypeError::new_err(msg))
                 }
             }
         }
     }
 }
-
-// --- IdentPrefix -----------------------------------------------------------
-
-/// The prefix of a prefixed identifier.
-#[pyclass(module = "fastobo.id")]
-#[derive(Clone, ClonePy, Debug, Eq, Hash, OpaqueTypedef, Ord, PartialEq, PartialOrd)]
-#[opaque_typedef(derive(FromInner, IntoInner, AsRefInner))]
-pub struct IdentPrefix {
-    inner: ast::IdentPrefix,
-}
-
-impl IdentPrefix {
-    pub fn new(prefix: ast::IdentPrefix) -> Self {
-        Self { inner: prefix }
-    }
-}
-
-impl Display for IdentPrefix {
-    fn fmt(&self, f: &mut Formatter) -> FmtResult {
-        self.inner.fmt(f)
-    }
-}
-
-impl ToPyObject for IdentPrefix {
-    fn to_object(&self, py: pyo3::Python<'_>) -> PyObject {
-        self.inner.as_str().to_object(py)
-    }
-}
-
-#[pymethods]
-impl IdentPrefix {
-    /// Create a new `IdentPrefix` instance.
-    ///
-    /// Arguments:
-    ///     value (`str`): the unescaped representation of the prefix.
-    #[new]
-    fn __init__(value: String) -> Self {
-        Self::new(ast::IdentPrefix::new(value))
-    }
-
-    /// `str`: the escaped representation of the identifier.
-    #[getter]
-    pub fn escaped(&self) -> PyResult<String> {
-        Ok(self.inner.to_string())
-    }
-
-    /// `str`: the unescaped representation of the identifier.
-    #[getter]
-    pub fn unescaped(&self) -> PyResult<&str> {
-        Ok(self.inner.as_str())
-    }
-}
-
-#[pyproto]
-impl PyObjectProtocol for IdentPrefix {
-    fn __repr__(&self) -> PyResult<PyObject> {
-        let gil = Python::acquire_gil();
-        let py = gil.python();
-        let fmt = PyString::new(py, "IdentPrefix({!r})").to_object(py);
-        fmt.call_method1(py, "format", (self.inner.as_str(),))
-    }
-
-    fn __str__(&'p self) -> PyResult<&'p str> {
-        Ok(self.inner.as_str())
-    }
-}
-
-// --- IdentLocal ------------------------------------------------------------
-
-/// The local component of a prefixed identifier.
-#[pyclass(module = "fastobo.id")]
-#[derive(Clone, ClonePy, Debug, Eq, Hash, OpaqueTypedef, Ord, PartialEq, PartialOrd)]
-#[opaque_typedef(derive(FromInner, IntoInner))]
-pub struct IdentLocal {
-    inner: ast::IdentLocal,
-}
-
-impl IdentLocal {
-    pub fn new(local: ast::IdentLocal) -> Self {
-        Self { inner: local }
-    }
-}
-
-impl Display for IdentLocal {
-    fn fmt(&self, f: &mut Formatter) -> FmtResult {
-        self.inner.fmt(f)
-    }
-}
-
-#[pymethods]
-impl IdentLocal {
-    /// Create a new `IdentLocal` instance.
-    #[new]
-    fn __init__(value: String) -> Self {
-        Self::new(ast::IdentLocal::new(value))
-    }
-
-    /// `str`: the escaped representation of the identifier.
-    #[getter]
-    fn get_escaped(&self) -> PyResult<String> {
-        Ok(self.inner.to_string())
-    }
-
-    /// `str`: the unescaped representation of the identifier.
-    #[getter]
-    fn get_unescaped(&self) -> PyResult<&str> {
-        Ok(self.inner.as_str())
-    }
-}
-
-#[pyproto]
-impl PyObjectProtocol for IdentLocal {
-    fn __repr__(&self) -> PyResult<PyObject> {
-        let gil = Python::acquire_gil();
-        let py = gil.python();
-        let fmt = PyString::new(py, "IdentLocal({!r})").to_object(py);
-        fmt.call_method1(py, "format", (self.inner.as_str(),))
-    }
-
-    fn __str__(&'p self) -> PyResult<&'p str> {
-        Ok(self.inner.as_ref())
-    }
-}
