@@ -2,7 +2,6 @@ use std::ops::Deref;
 
 use pyo3::ffi::PyObject;
 use pyo3::AsPyPointer;
-use pyo3::AsPyRef;
 use pyo3::Py;
 use pyo3::PyRef;
 use pyo3::PyTypeInfo;
@@ -47,3 +46,7 @@ pub trait AbstractClass: PyClass {
 }
 
 pub trait FinalClass: PyClass {}
+
+// ---
+
+pub type Hasher = std::collections::hash_map::DefaultHasher;
